@@ -39,8 +39,12 @@ from typing import Dict, Set
 
 from xdis.version_info import IS_GRAAL, IS_PYPY, version_tuple_to_str
 
-PYPY3_MAGICS = (48, 64, 112, 160, 192, 240, 244, 256, 336, 384)
-GRAAL3_MAGICS = (21150, 21280)
+PYPY3_MAGICS = (48, 64, 112, 160, 192, 240, 244, 256, 336, 384, 416)
+GRAAL3_MAGICS = (21150, 21280, 21290)
+RUSTPYTHON_MAGICS = (
+    12641, # RustPython 3.13
+    12897  # RustPython 3.12
+)
 
 
 def add_magic_from_int(magic_int, version):
@@ -456,12 +460,17 @@ add_magic_from_int(240, "3.7pypy")  # PyPy 3.7.9-beta0
 add_magic_from_int(256, "3.8pypy")  # PyPy 3.8.15
 add_magic_from_int(336, "3.9pypy")  # PyPy 3.9.15, PyPy 3.9.17
 add_magic_from_int(384, "3.10pypy")  # PyPy 3.10.12
+add_magic_from_int(416, "3.11.13pypy")  # PyPy 3.11.13
+
+add_magic_from_int(12897, "3.12.0rust")  # RustPython 3.12.0
+add_magic_from_int(12641, "3.13.0rust")  # RustPython 3.13.0
 
 # NOTE: This is JVM bytecode not Python bytecode
 add_magic_from_int(21150, "3.8.5Graal")
 
 # Graal Java 21.0.2
 add_magic_from_int(21280, "3.10.8Graal")
+add_magic_from_int(21290, "3.11.7Graal")
 
 
 add_magic_from_int(1011, "2.7.1b3Jython")  # jython
@@ -554,13 +563,13 @@ add_canonic_versions(
 add_canonic_versions(
     "3.9 3.9.0 3.9.1 3.9.2 3.9.3 3.9.4 3.9.5 3.9.6 3.9.7 3.9.8 3.9.9 3.9.10 3.9.11 "
     "3.9.12 3.9.13 3.9.14 3.9.14 3.9.15 3.9.16 3.9.17 3.9.18 3.9.19 3.9.10pypy 3.9.11pypy 3.9.12pypy "
-    "3.9.15pypy 3.9.16pypy 3.9.0b5+ 3.9.17 3.9.18 3.9.19 3.9.20",
+    "3.9.15pypy 3.9.16pypy 3.9.0b5+ 3.9.17 3.9.18 3.9.19 3.9.20 3.9.21 3.9.22 3.9.23",
     "3.9.0beta5",
 )
 
 add_canonic_versions(
     "3.10 3.10.0 3.10.1 3.10.2 3.10.3 3.10.4 3.10.5 3.10.6 3.10.7 3.10.8 3.10.9 "
-    "3.10.10 3.10.11 3.10.12 3.10.13 3.10.14 3.10.15 3.10.16 3.10.17",
+    "3.10.10 3.10.11 3.10.12 3.10.13 3.10.14 3.10.15 3.10.16 3.10.17 3.10.18",
     "3.10.0rc2",
 )
 
@@ -568,17 +577,17 @@ add_canonic_versions("3.10.13Graal", "3.10.8Graal")
 
 add_canonic_versions(
     "3.11 3.11.0 3.11.1 3.11.2 3.11.3 3.11.4 3.11.5 3.11.6 3.11.7 3.11.8 3.11.9 3.11.10 "
-    "3.11.11 3.11.12",
+    "3.11.11 3.11.12 3.11.13",
     "3.11a7e",
 )
 
 add_canonic_versions(
-    "3.12 3.12.0 3.12.1 3.12.2 3.12.3 3.12.4 3.12.5 3.12.6 3.12.7 3.12.8 3.12.9 3.12.10",
+    "3.12 3.12.0 3.12.1 3.12.2 3.12.3 3.12.4 3.12.5 3.12.6 3.12.7 3.12.8 3.12.9 3.12.10 3.12.11",
     "3.12.0rc2",
 )
 
 add_canonic_versions(
-    "3.13 3.13.0 3.13.1 3.13.2 3.13.3",
+    "3.13 3.13.0 3.13.1 3.13.2 3.13.3 3.13.4 3.13.5 3.13.6 3.13.7",
     "3.13.0rc3",
 )
 
